function sDictionary = Dictionary(sDetails,szSpace,dictionary)
%sDictionary = Dictionary(type,szSpace)


%[mDictionary, vSize, mDictionaryKron] = Dictionary(type, sz1DSpace)
% Dictionary Constructs and returns the requested dictionary.
%
% Returns the Kronecker product of either:
% 1) The discrete cosine atoms spanning \(\mathbb{R}^{\mbox{sz1DSpace}}\) with themselves or,
% 2) The combined dictionary \([D_{a}, D_{b}]\) with itself where:
%   \(D_{a}\) - Normalized redundancy \(2\) discrete cosine dictionary.
%   \(D_{b}\) - Dirac basis.
%
% Usage:    [mDictionaryKron, mDictionary, vSize] = Dictionary( type, sz1DSpace );
%   
% Inputs:
%   type        DCT - Basis of discrete cosines for the space \(\mathbb{R}^{\mbox{sz1DSpace}}\).
%               RDCTDB - Dictionary \([D_{a}, D_{b}]\).
%   sz1DSpace   Dimension of atoms before applying the Kronecker product.
%
% Output:
%   mDictionaryKron Matrix containing the \(2\)D dictionary.
%   mDictionary     Matrix containing the \(1\)D dictionary.
%   vSize           Vector containing the sizes of each 1D dictionary. 

%% Main Routine
%sDetails = struct('name',{'DCT','Hats','Haar'},'parameters',{2,1,2});

% Check sDetails has the correct fields
if ~isfield(sDetails,'name') || ~isfield(sDetails,'parameters')
    error('The details structure does not have the correct format!');
end

% Allows for removal of boundry functions
% < < - - #
    if nargin < 3
        dictionary = 1;
    end
% - - > > #
% nSubDictionaries = size(sDetails,2);

%% Initialize sDictionary
sDictionary.sDetails = sDetails;
sDictionary.description = [];
sDictionary.mDictionary = [];
sDictionary.vSize = [];
sDictionary.nSubDictionaries = size(sDetails,2);


for i = 1:sDictionary.nSubDictionaries
    
    if strcmp(sDetails(i).name,'DCT')
        
       % Check redundancy is one or greater
       redundancy = sDetails(i).parameters;
       if isnumeric(redundancy) && redundancy == 1
           
           mDictLoop = NormDict(DCos(szSpace, szSpace*redundancy, redundancy));
           
       elseif isnumeric(redundancy) && redundancy > 1                     
        
           mDictLoop = NormDict(DCos(szSpace, szSpace*redundancy, redundancy));
           
       elseif isnumeric(redundancy) && redundancy < 0
           
           % Remove half the redundant cosines
           redundancy = -1*redundancy;
           mDictLoop = NormDict(DCos(szSpace, szSpace*redundancy/2, ...
               redundancy));
           
       else
           
           error('Incorrect value for reduncancy of the DCT');

       end
       
    elseif strcmp(sDetails(i).name,'Hats')
        
        % Check positive length of support
        lSupport = sDetails(i).parameters;
        if isnumeric(lSupport) && lSupport > 0
            
            vPrototype = GenerateTrapezium(lSupport,1);
            mDictLoop = TranslatePrototype(vPrototype,szSpace,dictionary);
            
        else
            
            error('Incorrect value for length of support of the Hat function');

        end
        
    elseif strcmp(sDetails(i).name,'Haar')
        
        % Check positive length of support
        lSupport = sDetails(i).parameters;
        if isnumeric(lSupport) && lSupport > 0
            
            vPrototype = GenerateTrapezium(lSupport,lSupport);
            mDictLoop = TranslatePrototype(vPrototype,szSpace,dictionary);
            
        else
            
            error('Incorrect value for length of support of the Haar function');

        end    
        
    elseif strcmp(sDetails(i).name,'Cubic')
        
        % Check positive level
        level = sDetails(i).parameters;
        if isnumeric(level) && level > 0
            
            % Calculate the whole dictionary mDict
            nFunctions = szSpace *2;
            partition = 1:nFunctions;                        
            mDict = CutDic(partition,4,nFunctions,level);
            % Pick the whole symetric function - should be at position 4
            lSupport = level*4-1;
            if lSupport > szSpace
                
                error('Length of atoms support exceeds the size of space!');
                
            end
                
            vPrototype = mDict(2:2+lSupport-1,4);
                       
            mDictLoop = TranslatePrototypeW(vPrototype,szSpace,dictionary);
            
        else
            
            error('Incorrect value for level used to generate cubic spline function');

        end         
        
    else
        
        error('Unrecognised function: %s!',sDetails(i).name);
        
    end
    
    sDictionary.description = [sDictionary.description, sDetails(i).name,...
        num2str(sDetails(i).parameters)];
    sDictionary.mDictionary = [sDictionary.mDictionary, mDictLoop];
    sDictionary.vSize = [sDictionary.vSize, size(mDictLoop,2) ];
    
end

% Normalize the atoms
sDictionary.mDictionary = NormDict(sDictionary.mDictionary);