function imagePathNew = ImageTo16BitRaw(imagePathOld)

    % Converts an image file to the correct format to be used in the 
    % simulation.
    % 1) The dimensions should be a multiple of blockWidth and blockHeight
    % 2) The dynamic range should be 2^16
    % 3) The image should be in .raw format created using SaveImageRaw
 
%     % Check image format
%     % <<--
%         CheckRGBGrey(imagePathOld);
%     % -->>
%     
%     % Read in image file
%     % <<--
%         mImage = double(imread(imagePathOld));
%     % -->>
    
    % Read in file and covert to array of doubles
    % <<--
        mImage = LoadImage(imagePathOld);
    % -->>

    % Rescale to have dynamic range of 2^16
    % <<--
        mImageRescaled = Rescale(mImage,0,2^16-1);
    % -->>

    % Resize if necessary
    % <<--
        BLOCKWIDTH = 8;
        BLOCKHEIGHT = 8;
        mImageRescaled = CheckBlocking(mImageRescaled, BLOCKWIDTH, ...
            BLOCKHEIGHT);
    % -->>

    % Change file extension to .raw
    %   First get the entire path to the image in case imagePath old is on
    % the current search path and doesn't include the whold path.
    % <<--
        imagePathOld = GetFullPath(imagePathOld);
        imagePathNew = ChangeExtension(imagePathOld,'.raw');
    % -->>

    % Write raw image file
    % <<--
        SaveImageRaw(mImageRescaled,imagePathNew,'uint16');
    % -->>

end

function fullPath = GetFullPath(fileName)

    % Returns the full path to the file
    
    % Function uses which to return the path to the matlab file.
    % If which cannot find the file then assume that fileName is a path
    % to the file and use that.
    
    % Use which to see if file is on the matlab path or in the current
    % directory.

    % <<--
        fullPath = which(fileName);
    % -->>
    
    % If file is not on matlab path or in the current directory assume
    % fileName is the path to the file.
    %   which returns an empty variable if the file cannot be found, test
    % for this below.
    % <<--
        if isempty(fullPath)
            fullPath = fileName;
        end
    % -->>
    
end

%{
function CheckRGBGrey(imagePath)

    % Checks the image info ColourType field for truecolor or grayscale

    % Get the image color type info
    % <<--
        imageInfo = imfinfo(imagePath);
        colourType = imageInfo.ColorType;
    % -->>

    % Throw error if ColourType field is not truecolor, grayscale or
    % indexed
    % <<--
        if ~strcmp(colourType,'truecolor') && ~strcmp(colourType,'grayscale')
            error('Image Color Type should be truecolor of grayscale');
        end
    % -->>

end
%}

function mImage = LoadImage(imagePathOld)

    % Reads in an image.  If image is indexed converts to RGB or grey

    % Get image colourType
    % <<--
        imageInfo = imfinfo(imagePathOld);
        colourType = imageInfo.ColorType;       
    % -->>
    
    % Load and convert if indexed
    % <<--
        if strcmp(colourType,'indexed')
            
            % Read image and colour map
            [mImageInd map] = imread(imagePathOld);
            
            % Check for grey of rgb colour map
                if (sum(map(:,1) - map(:,2)) > 0)...
                    || (sum(map(:,1) - map(:,3)) > 0)
                    
                    % Convert to RGB
                    % <<--
                        mImage = ind2rgb(mImageInd,map);
                    % -->>
                else
                    % Convert to grey
                    % <<--
                        mImage = ind2gray(mImageInd,map);
                    % -->>                                       
                end
                    
        elseif strcmp(colourType,'truecolor') || ...
                strcmp(colourType,'grayscale')
             
            % Read in image
            mImage = imread(imagePathOld);
            
        end
    % -->>
    
    % Convert unsigned image matrix to double
    % <<--
        mImage = double(mImage);
    % -->>

end


function newPath = ChangeExtension(oldPath,newExtension)

    % Changes the file extension to newExtension
    
    % Split oldPath
    [pathString,fileName,extension] = fileparts(oldPath);
    
    % Concatinate existing path and file name with newExtension
    newPath = [pathString,'/',fileName,newExtension];
    
end