function [mApprox vIChosenC vIChosenR mBiorth mCoeff mOrth vISepAtom] = ...
    OMPMlSepMod(mImage,mColumnDict,mRowDict,tol,vWeights,No,vIndC,vIndR)
% OMP2DMl Orthogonal Matching Pursuit in 2D for multiple signals
%
%It creates an atomic decomposition of one or more 2D signal simultaneously
%using OMP criterion and assuming separable dictionaries.
%You can choose a tolerance, the number of atoms to take in 
% or an initial subspace to influence the OMP algorithm. 
% 
%  
% Usage:    [H,Di1,Di2, beta, c,Q ]= OMP2D(f, Dx, Dy, tol,pp, No, indx,indy);
%           [H] = OMP2D(f,Dx,Dy); variables tol, No, can also be []
%                 
%
% Inputs:
%   f       analyzing multi 2D signals (Images) rows x colums x n (number of them)
%   Dx      dictionary of normalized atoms (w.r.t. Image rows)
%   Dy      dictionary of normalized atoms (w.r.t. Image columns)
%   tol     desired tolerance for approximation default tol=0.03*norm(f(:,:,1),'fro');
%   pp      nonnegative weights for a convex combination of signals sum(p)=1
%   No      (optional) maximal number of atoms to choose, if the number of chosen atoms
%           equals to No, routine will stop (default numel(f))
%   indx    (optional) indices for an initial subspace; They operate as indx(k),indy(k) 
%   indy    (optional) indices determining  the initial subspace (as above)
%
% Outputs:
%   H       Approximation of the f array 
%   Di1     indices of selected atoms w.r.t the original Dx
%   Di2     indices of selected atoms w.r.t the original Dy
%   beta    biorthogonal atoms corresponding to the 2D selected atoms 
%   c       coefficients of the atomic decomposition, for each signal 
%   Q       orthogonal 2D basis for the selected subspace
%  
%   
% The implementation of OMP method is based on  Gram-Schmidt orthonormalization and 
% adaptive birothogonalization, as proposed in Ref 2 below.
%
%  References:
%
%  1-Y.C. Pati, R. Rezaiifar, and P.S. Krishnaprasad, "Orthogonal matching pursuits:
%  recursive function approximation with applications to wavelet decomposition", in Proceedings of
%  the 27th Asilomar Conference on Signals, Systems and Computers, 1993.
%   
%  2-L. Rebollo-Neira and D. Lowe, "Optimized Orthogonal Matching Pursuit Approach", IEEE
%  Signal Processing Letters, Vol(9,4), 137-140, (2002). 
%
% See also OMP2D, MP2D, SPMP2D, OMP, MP, SPMP and all Pursuit routines at the link below
%
% http://www.nonlinear-approx.info/

% Size vars from manditory inputs
    % Image dimensions
        [nRows nColumns nColours] = size(mImage);
        nPixelsPlain = nRows * nColumns;
    % - - > >
    
    % Dictionary dimensions
        nColumnAtoms = size(mColumnDict,2);
        nRowAtoms = size(mRowDict,2);
        nSepAtoms = nRowAtoms * nColumnAtoms;
    % - - > > 
% - - > > #

% Check optional inputs
    if (nargin<8) || (isempty(vIndC)==1) 
        vIndC=[];
    end
    if (nargin<7) || (isempty(vIndR)==1) 
        vIndR=[];
    end   
    if (nargin<6) || (isempty(No)==1)   
        No=nPixelsPlain;
    end
    if (nargin<5) || (isempty(vWeights)==1)   
        equalWeights = 1;
        %vWeights(1:nColours)=1/nColours;
    else
        % Check for equal weights assume add to one
%             if sum(vWeights == 1/nColours) == nColours
%                 equalWeights = 1;
%             else
%                 equalWeights = 0;
%             end
        % - - > >
        
        % vWeights with 1 at the first position indicates constant weights
        if vWeights(1) == 1
            equalWeights = 1;
        else
            equalWeights = 0;
        end        
        
    end
    if (nargin<4) || (isempty(tol)==1)  
        tol=0.03*norm(mImage(:,:,1),'fro');
    end
% - - > >

% Control parameters
     % Threshold for coefficients
        tol2=1e-10;   %0.0001  %1e-5
    % - - > >
    % Maximal number of function in sub-dictionary
        maxIter=min(No,nSepAtoms);
    % - - > >
% - - > > #

% Initialize loop structures
    % Initialize to max possible size
    % Use coder.nullcopy(zeros(1,N));
        vIChosenC = zeros(1,maxIter);
        vIChosenR = zeros(1,maxIter);
        vISepAtom = zeros(1,maxIter);
        mOrth = zeros(nPixelsPlain,maxIter);
        mAtom = zeros(nPixelsPlain,maxIter);
        mBiorth = zeros(nPixelsPlain,maxIter);
        vNormOrth = zeros(1,maxIter);
    % - - > >
    
    mRemove = zeros(nRows,nColumns,nColours);
    
    % Vector to reduce execution time used for calculating mRemove
        mImageVec = reshape(mImage,nPixelsPlain,nColours);
        if ~equalWeights
            for i = 1:nColours
                mImageVec(:,i) = mImageVec(:,i) * vWeights(i);
            end
        end
    % - - > >
% - - > > #

% Process initial indexes if provided
    numind=numel(vIndC); % Assume both provided
    if numind
        % Validate initial indexes
        %   Testing boundries is quicker than individual indexes
            ValInd(vIndC,1,nColumnAtoms);
            ValInd(vIndR,1,nRowAtoms);
        % - - > >

        % Assign to return vectors
            vIChosenC(1:numind) = vIndC(1:numind);
            vIChosenR(1:numind) = vIndR(1:numind);
            
            % Calculate seperable atoms index
                vISepAtom(1:numind) = sub2ind([nColumnAtoms,nRowAtoms],vIndC,...
                    vIndR);
            % - - > > 
        % - - > >
    end
% - - > > #



% In loop vars
    nReOrth = 1;
    mResidule = mImage;
    if ~equalWeights
        for i = 1:nColours
            mResidule(:,:,i) = mResidule(:,:,i)*vWeights(i);
        end
        % To perfom the same as previous algorithm
        % Remove weighting from residule ? nColours
        vWeightsSquared = nColours*vWeights.^2;
%         vWeightsSS = 
    end
% - - > > #

%===============================
% Main algorithm: at kth iteration
%===============================
for iIter = 1:maxIter    
    if iIter <= numind 
        
        iCAtom = vIChosenC(iIter);
        iRAtom = vIChosenR(iIter);
        
    else

        % Calculate inner products for each plain
        %   Cannot use average due to subativity of abs
            cc=zeros(nColumnAtoms,nRowAtoms);
            for iColour=1:nColours
                cc = cc + abs(mColumnDict'*...
                    mResidule(:,:,iColour)*mRowDict);
            end
        % - - > >
            
        % Locate maximum inner product
        %   Get indexes in local dicitonaries and index of seperable atom.
            [maxCoeff iSepAtom] = max(cc(:));
            iRAtom = ceil(iSepAtom/nColumnAtoms);            
            iCAtom = iSepAtom - ((iRAtom-1)*nColumnAtoms);
        % - - > >

        % Stopping criterion (coefficient)
        if maxCoeff < tol2 
            iIter = iIter - 1;
            fprintf('Stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',tol2);
            break;
        else
            % Store chosen atoms
                vIChosenC(iIter) = iCAtom;
                vIChosenR(iIter) = iRAtom;
                vISepAtom(iIter) = iSepAtom;
            % - - > >
        end
    % - - > >
    end  

    % Form seperable atom
    %   Outer product should be faster than kron
    try
        newAtom = reshape(mColumnDict(:,iCAtom)*mRowDict(:,iRAtom)',...
            nPixelsPlain,1);
    catch
        debug = 1;
    end
    % - - > > #
    
    % Stored for calculation of biorthogonal atoms    
        mAtom(:,iIter) = newAtom;
    % - - > > #
    
    % Orthogonal atoms step
        % Start building next orthogonal atom
            mOrth(:,iIter) = newAtom;
        % - - > >

        if iIter > 1

            % Orthogonalization of Q(:,k)  w.r.t Q(:,1),..., Q(:,k-1) 
            %   Again use matrix mult for speed - slow in MATLAB fast in c
                for zi=1:nReOrth+1
                    mOrth(:,iIter) = mOrth(:,iIter) - (mOrth(:,1:iIter-1)*...
                        (mOrth(:,1:iIter-1)'*mOrth(:,iIter)));
                end
            % - - > >            
        end

        % Normalize orthogonal atom
            normOrth = norm(mOrth(:,iIter));
            mOrth(:,iIter) = mOrth(:,iIter)/normOrth;
            % Required for calculating biorthogonal atoms
                vNormOrth(iIter) = normOrth;
            % - - > >
        % - - > >
    % - - > > #
    
    % Calculate component of residule in mOrth
        for iColour = 1:nColours
            mRemove(:,:,iColour) = reshape( (mImageVec(:,iColour)' * ...
                mOrth(:,iIter)) * mOrth(:,iIter) ,nRows,nColumns );
        end 
    % - - > > #
    
    % Update the residule
        mResidule = mResidule - mRemove;
    % - - > > #    
    
    % Update the norm of the residue
        if  equalWeights
            ssResidule = sum(mResidule(:).^2);
        else
            ssResidule = sum(reshape((sum(sum(mResidule.^2,1),2)),...
                1,nColours)./vWeightsSquared);
        end
%         normResidule = sum(reshape(sqrt(sum(sum(mResidule.^2,1),2)),...
%             nColours,1));
%         normResidule = sum(mResidule(:).^2);
%         normResidule = norm(mResidule(:));
    % - - > > #
    
    % Stopping criterion (distance)
    if all(tol~= 0) && all(ssResidule < tol)
        break;
    end
end

% Calculate biorth here for speed
    mBiorth(:,1) = mOrth(:,1)/vNormOrth(1);
    for i = 2:iIter
        mBiorth(:,1:i-1) = mBiorth(:,1:i-1)-mOrth(:,i)*((mAtom(:,i)'...
            *mBiorth(:,1:i-1))/vNormOrth(i));
        mBiorth(:,i) = mOrth(:,i)/vNormOrth(i);
    end
% - - > > #

% Calculate the coefficients
    mImageVec = reshape(mImage,nPixelsPlain,nColours);
    mCoeff = mImageVec'*mBiorth;
% - - > >

% Remove weighting from the residule
    if ~equalWeights
        for i = 1:nColours
            mResidule(:,:,i) = mResidule(:,:,i)/vWeights(i);
        end
    end
% - - > > #

% Calculate the approximation
    mApprox = mImage - mResidule;
% - - > > #
    
% Reduce the size of large arrays
    vIChosenC = vIChosenC(1:iIter);
    vIChosenR = vIChosenR(1:iIter);
    mCoeff = mCoeff(:,1:iIter);
    mOrth = mOrth(:,1:iIter);
    mBiorth = mBiorth(:,1:iIter);
    vISepAtom = vISepAtom(1:iIter);
% - - > > #


%Laura REBOLLO-NEIRA 2011, based on the OMP implementation copyright below
%
%Copyright (C) 2006 Miroslav ANDRLE and Laura REBOLLO-NEIRA
%
%This program is free software; you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation; either 
%version 2 of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
%without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with this program;
%if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%Boston, MA  02110-1301, USA.

function ValInd(vInd,minInd,maxInd)

    % Use max and min
        if min(vInd) < minInd  || max(vInd) > maxInd
            error('Demanded index is out of dictionary');
        end
    % - - > > #
    


