#ifndef OMP2D_H
#define OMP2D_H

#ifndef output
#define output 1
#endif

#ifndef linux
#define linux 0
#endif

#if linux
#include <time.h>
#include <sys/time.h>
#endif

#define msg mexPrintf
#include "../../Common/PursuitFunctions.h"
#include "../../Common/MatrixOperations.h"
#include <algorithm>

int OMP2D(double *mApproxImage, double *mImage, double *mDictionaryY, double *mDictionaryX, double tolerance1, unsigned long szSpaceX, unsigned long szSpaceY, 
        unsigned long nAtomsX, unsigned long nAtomsY, double *vIYBig, double *vIXBig, double *vIXYBig, bool bigIndex, double *mBeta, double *vCn, double *mOrthogonal, unsigned long No, 
        double *indY, double *indX, unsigned long numInd, unsigned long &nChosen)
{

    const char name[] = "OMP2D";
    
    #if output
        #if linux
            /* tic; */
            timeval start;
            gettimeofday(&start, 0);
        #endif
    #endif
    
    unsigned long szSpaceYX = szSpaceY*szSpaceX;
    /* More validation */        
    unsigned long nIterations = min(szSpaceYX,nAtomsY*nAtomsX);
    if (No < 0  || No > nIterations)
    {
        msg("The minimum number of atoms %i must be greater than 0 and less than %i!\n",No,nIterations);
        return 1;
    }else if (No != 0)
    {          
        nIterations = min(nIterations, No);
    }
    
    /* Check positive tol */
    if ( tolerance1 < 0 )
    {
        msg("Tol is %g is should be positive!\n",tolerance1);
        return 1;
    }
        
    /*  %atoms having smaller norm than tol1 are supposed be zero ones
     *  tol1=1e-7; %1e-5
     *  %threshold for coefficients
     *  tol2=1e-10;   %0.0001  %1e-5
     */
    const double tolerance2 = 1e-10;
    const double tolerance3 = 1e-7;
    
    /* Re=f;
     */
    double *mResidule = new double[szSpaceYX];
    CopyElements(mImage,mResidule,szSpaceYX);  
    
    /* If index arrays are provided validate them*/
    if(numInd != 0)
    {
        // ValidateIndex throws error
        ValidateIndex(indX, nAtomsX, numInd);
        ValidateIndex(indY, nAtomsY, numInd);        
    }
      
    unsigned int minCn = 0;
    
    /*  fprintf('\n%s is running for tol=%g, tol1=%g and tol2=%g.\n',name,tol,tol1,tol2);
     *  fprintf('tol  -> required precision, distance ||f-f_approx||\n')
     *  fprintf('tol1 -> atoms having smaller norm are supposed to be zero ones.\n');
     *  fprintf('tol2 -> algorithm stops if max(|<f,q>|/||q||)<= tol2.\n');
     */
    #if output
        msg("\n%s is running for tol=%g, tol1=%g and tol2=%g.\n",name,tolerance1,tolerance3,tolerance2);
        msg("tol  -> required precision, distance ||f-f_approx||\n");
        msg("tol1 -> atoms having smaller norm are supposed to be zero ones.\n");
        msg("tol2 -> algorithm stops if max(|<f,q>|/||q||)<= tol2.\n");
    #endif
    /*************************************************************/
    /********************* Start of main routine *****************/
    /*************************************************************/

    /* Declare here as needed after the loop exits */
    unsigned long k = 0;
    double ssResidule = 0;
    double * chosenAtom = new double[szSpaceYX];
            
    for ( k = 0; k < nIterations; k++)
    {    
        /* Declare variables that will only be used inside the loop here */        
        unsigned long iNewAtom = k*szSpaceYX;
        unsigned long iChosenAtomY = 0;
        unsigned long iChosenAtomX = 0;
        unsigned long iChosenAtomXY = 0;

        /* if k <= numind */
        if ( k+1 <= numInd )
        {
            iChosenAtomY = static_cast<int>(indY[k])-1;
            iChosenAtomX = static_cast<int>(indX[k])-1;
            iChosenAtomXY = iChosenAtomX*nAtomsY+iChosenAtomY;
            /* Check that no duplicate entries in indY and indX, as sorted check next entry in the arrays */
            /* First check we are not at the end of the dictionary */
            if(( k+1 != numInd))
            {
                /* If the atom at this index is equal to the atom at the next index */
                if ( (iChosenAtomY == (static_cast<int>(indY[k+1])-1)) || (iChosenAtomX == (static_cast<int>(indX[k+1])-1)))
                {
                    msg("Duplicate entry in initial subspace indices!\n");
		    delete[] mResidule;
		    mResidule = 0;
		    delete[] chosenAtom;
		    chosenAtom = 0;
                    return 1;
                }
            }
            
        }else{
            /* Choose the next atom from the unselected atoms. */
            /*  for p=k:N, %it can be done without for
             *      cc(p)=abs(Re*D(:,p));
             *  end      
             *  [max_c,q]=max(cc);      
             */    
            double maxValue = 0;
            ChooseAtomOMP2D(mDictionaryY, mDictionaryX, mResidule, nAtomsY, nAtomsX, szSpaceY, szSpaceX, maxValue, 
                    iChosenAtomY, iChosenAtomX, iChosenAtomXY);
            
            /* Stopping criterion (coefficient)
             * MATLAB CODE
             * if max_c<tol2
             *      fprintf('%s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',name,tol2);
             *      break;
             * end
             */
            if ( maxValue < tolerance2 )
            {
                #if output
                    msg("%s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n",name,tolerance2);
                #endif
                break;    
            }
        }

		/* Populate the matrices with the new atoms
         * MATLAB CODE
         *    Di1(k)=q(1);
         *    Di2(k)=q(2);
         */
        /* Add 1 as MATLAB index's */
        vIXBig[k] = iChosenAtomX + 1;
        vIYBig[k] = iChosenAtomY + 1;
		if (bigIndex)
		{
			vIXYBig[k] = iChosenAtomXY + 1;
		}

        /* Get the chosen atom */
		KronAtom(&mDictionaryX[iChosenAtomX*szSpaceX],&mDictionaryY[iChosenAtomY*szSpaceY],szSpaceX,szSpaceY,chosenAtom);

        /* if k>1 */
        if ( k > 0 )
        {
            /*  for p=1:k-1
             *      Q(:,k)=D(:,k)-(Q(:,p)'*D(:,k))*Q(:,p); %orthogonalization
             *  end 
             */
            OrthogonalizeOMP(mOrthogonal, chosenAtom, szSpaceYX, k);

            /* Re-orthogonalization of Q(:,k)  w.r.t Q(:,1),..., Q(:,k-1)
             * MATLAB CODE
             *  for zi=1:zmax      
             *      for p=1:k-1
             *          Q(:,k)=Q(:,k)-(Q(:,p)'*Q(:,k))*Q(:,p);
             *   	end	   
             *  end
             */     
             Reorthognalize(mOrthogonal,szSpaceYX,k,2);
        }              
        
        /*  if k==1 
         *      Q(:,k)=D(:,k); 
         *  end
         */
        if ( k == 0 )
        {
            CopyElements( chosenAtom, mOrthogonal, szSpaceYX );
        }
        
        /* Normalize atom Q(:,k)
         * MATLAB CODE
         * nork=norm(Q(:,k));
         * Q(:,k)=Q(:,k)/nork; %normalization
         */      
        double normKthAtom = Normalize(&mOrthogonal[iNewAtom],szSpaceYX,0);
         
        /* Compute biorthogonal functions beta
         *   if k>1
         *      beta=beta-Q(:,k)*(D(:,k)'*beta)/nork; 
         *  end	
         *  beta(:,k)=Q(:,k)/nork;          % kth biorthogonal function 
         */
        CalcBiorthogonal(mBeta, chosenAtom, &mOrthogonal[iNewAtom], normKthAtom, szSpaceYX, k);
             
        /* %%%%computes de residum%%%%%    
         * h=f(:)'*Q(:,k)*Q(:,k)';
         * Re=Re(:)-h';
         * nore=(norm(Re))^2*(delta);
         * Re=reshape(Re,Lx,Ly);        
         */
        /* Works the same as in 1D as all matrices are stored as 1D arrays */
        CalcResiduleOMP(mResidule, mImage, &mOrthogonal[k*szSpaceYX], szSpaceYX);       
                
        /* Calculate the norm of the residule */
        /*  MATLAB CODE
         *  Re = norm(f'-D(:,1:k)*(f*beta)')
         */  
         ssResidule  = Norm(mResidule, szSpaceYX);

		 /* Sum Squared Error */
		 ssResidule = pow(ssResidule,2);

         //normResidule = sqrt(RealInnerProduct( vResidule, vResidule, szSignal ));
        
         /* Stopping criteria (distance)
          * MATLAB CODE - We use the residule check with omp if this is the same
          * if (norm(f'-D(:,1:k)*(f*beta)')*sqrt(delta) < tol) && (tol~= 0)break;end;
          */
        if ( ssResidule < tolerance1 && tolerance1 != 0 && k+1 > minCn)
        {
            /* Break so will not increment k before exiting the loop */
            k += 1;
            break;                          
        }
    }
    
    if (k == 0)
	{
		/* Assume that zero of close to zero signal provided so return zero coefficient at position 1 */
		vCn[0] = 0;
        vIXBig[0] = 1;
        vIYBig[0] = 1;
		if (bigIndex)
		{
			vIXYBig[0] = 1;
		}
		/* Return zero block as that is the approximation*/
		AllocateElements(mApproxImage, 0, szSpaceYX);
		/* Populate beta(:,1) and Q(:,1) with zeros for speed so the dimensions are consistent */
		AllocateElements(mOrthogonal, 0, szSpaceYX);
		AllocateElements(mBeta, 0, szSpaceYX);
		/* As we want nChosen to be 1 */
		k =1;
	}else{
		/* c=f*beta; */
		MultiplyMatrixVector(mBeta, mImage, vCn, szSpaceYX, k, 'T');
        
		/* MATLAB Code
		 *  for n=1:id;
		 *      H=H+Dx2(:,n)*c(n)*Dy2(:,n)';
		 *  end
		 * Just subtract the residule for speed otherwise would be quicker to store all
		 * the chosen atoms and multiply them by mImage make a function so can be altered easily.
		 */ 
		/* mResidule = -mResidule + mImage */
		AddVectors1(mResidule, mImage, -1, szSpaceYX);
		/* mApproxImage = mResidule */
		CopyElements(mResidule,mApproxImage,szSpaceYX);
	}

    delete[] mResidule;
    mResidule = 0;
    delete[] chosenAtom;
    chosenAtom = 0;
    
    /*  MATLAB Code
     *  fprintf('From %d atoms in this dictionary %s has chosen %d atoms.\n',N,name,k);
     *  fprintf('\n%s lasted %g seconds\n',name,toc); 
     *  fprintf('\nNorm ||f-f_approx|| is %g. \n',normf);
     */
    #if output
        msg("From %d atoms in this dictionary %s has chosen %d atoms.\n",nAtoms,name,k);
        #if linux
            timeval finish;
            gettimeofday(&finish, 0);  
            msg("%s ran for %g seconds\n",name,((static_cast<double>(finish.tv_sec)*1e6+static_cast<double>(finish.tv_usec)) - 
                    (static_cast<double>(start.tv_sec)*1e6+static_cast<double>(start.tv_usec))) *1e-6);  
        #endif
        msg("\nNorm ||f-f_approx|| is %g. \n",normResidule);
    #endif
    
    nChosen = k;
    return 0;
}
#endif
