/*
 * Gateway to implementation of OMPMlSep in c++.
 * 
 * OMPMlSep - Orthogonal Matching Pursuit (with) Seperable (Dictionary) Multi
 * 
 * MATLAB CALL - Old call [Dnew,Di,beta,c,Q]=OMPMl(f,D,tol,pp,No,ind);
 *  [mApprox vIChosenC vIChosenR mBiorth mCoeff mOrth vISepAtom] = ...
 *      OMP2DMl(mImage,mColumnDict,mRowDict,tol,vWeights,No,vIndC,vIndR);
 *
 *  MATLAB Code
      % Size vars from manditory inputs
            % Image dimensions
                [nRows nColumns nColours] = size(mImage);
                nPixelsPlain = nRows * nColumns;
            % - - > >

            % Dictionary dimensions
                nColumnAtoms = size(mColumnDict,2);
                nRowAtoms = size(mRowDict,2);
                nSepAtoms = nRowAtoms * nColumnAtoms;
            % - - > > 
        % - - > > #

        % Check optional inputs
            if (nargin<8) || (isempty(vIndC)==1) 
                vIndC=[];
            end
            if (nargin<7) || (isempty(vIndR)==1) 
                vIndR=[];
            end   
            if (nargin<6) || (isempty(No)==1)   
                No=nPixelsPlain;
            end
            if (nargin<5) || (isempty(vWeights)==1)   
                vWeights(1:nColours)=1/nColours;
            end
            if (nargin<4) || (isempty(tol)==1)  
                tol=0.03*norm(mImage(:,:,1),'fro');
            end
        % - - > >
 */
#define output 0
#include "mex.h"
#include "matrix_operations.h"
#include "OMPMlSep_Mex.h"
#include "OMPMlSep.h"

namespace std {};
using namespace std;

void mexFunction(int nlhs, mxArray *plhs[],
    int nrhs, const mxArray *prhs[])
{
    /* Gateway from MATLAB */
    OMPMlSep_Mex(nlhs,plhs,nrhs,prhs);
}

static void OMPMlSep_Mex( int nlhs, mxArray *plhs[],
        int nrhs, const mxArray *prhs[])
{
    /* Validate input to function */
        
    /* Validate number of input (rhs) arguments*/    
    if((nrhs < 3) || (nrhs > 8))
    {
        mexErrMsgTxt("Wrong number of input arguments");
    }
    
    /* Check we are being passed real full and double variables */
    for ( int i = 0; i < nrhs; i++ )
    {        
        if ( mxIsComplex(prhs[i]) || mxIsClass(prhs[i],"sparse")|| 
                !mxIsDouble(prhs[i]) )
        {
            mexErrMsgTxt("Input must be double, real, full");
        }
    }
    
    /*
     * Get the manditory inputs.
     */   
    
    /* Get the 3D image
     */
    ulng nRows = 0;
    ulng nColumns = 0;
    ulng nColours = 0;
    double *mImage = Get3DImage(prhs[0],nRows,nColumns,nColours);  

   /* Get 1D Dictionary (horizontal coordinate) performs 
    * inner products with the columns of the residule
    */
   ulng nAtomsColumn = 0;
   double *mDictColumns = GetDict(prhs[1],nRows,nAtomsColumn);    
    
   
   /* Get 1D Dictionary (vertical coordinate) performs 
    * inner products with the rows of the residule 
    */
    ulng nAtomsRow = 0;
    double *mDictRows = GetDict(prhs[2],nColumns,nAtomsRow);
    
    /* Number of pixels
     *  Calculated here assuming validation is passes
     */
    ulng nPixelsPlain = nRows*nColumns;
    
    /* 
     * Check for the optional inputs
     */
    // <<---
        /* Get the numerical tolerance for approximation
         * MATLAB
         *  if (nargin<4) || (isempty(tol)==1)  
         *      tol=0.03*norm(mImage(:,:,1),'fro');
         *  end
         */
        double tolApprox = 0;  
        // If their is a variable and its dimensions are not zero retreive it
        if ((nrhs > 3) && !mxIsEmpty(prhs[3]))
        {
            tolApprox = GetDouble(prhs[3]);
        }else{
            tolApprox = 0.03*Matrix::VNorm(mImage,nPixelsPlain);
        }    
                                
        /* Get the weights vector
         * MATLAB
         *  if (nargin<5) || (isempty(vWeights)==1)   
         *      vWeights(1:nColours)=1/nColours;
		 *  Initialize array of one and set it to one to indicate
		 *  constant weights.
         *  end
         */
        dbl *vWeights = 0;
        if ((nrhs > 4) && !mxIsEmpty(prhs[4]))
        {           
            vWeights = GetVWeights(prhs[4],nColours);
        }else{
            //double weight = 1.0/static_cast<double>(nColours);
            //for (ulng i = 0; i < nColours; i++ )
            //{
            //    vWeightsStat[i] = weight;
            //}
            // Point to the first element of vWeightsStat
			dbl vWeightsStat[1];
			vWeightsStat[0] = 1.0;
            vWeights = &vWeightsStat[0];
        }  
                    
        /* Get maximum number of atoms in the approximation
         * MATLAB
         *  if (nargin<6)  | (isempty(No)==1)   
         *      No = nPixelsPlain;
         *  end
         */
        ulng maxNAtoms = 0;
        if ((nrhs > 5) && !mxIsEmpty(prhs[5]))
        {
            maxNAtoms = GetUlng(prhs[5]);
        }else{
            maxNAtoms = nPixelsPlain;
        }  
                
        /* Indexes for both initial subspaces should be provided */
		if (nrhs == 7)
		{
			mexErrMsgTxt("Initial indexes should be provided for both dictionaries!\n");
		}


       /* Get indx (optional) indices for an initial subspace;
        *  MATLAB
        *   if (nargin<8) || (isempty(vIndC)==1) 
        *       vIndC=[];
        *   end
        *   if (nargin<7) || (isempty(vIndR)==1) 
                vIndR=[];
            end   
		*/
        double *vIndDictColumns = 0;
		double *vIndDictRows = 0;
        ulng nInitialIndexes = 0;
        if (nrhs > 7)
        {
            vIndDictColumns = GetVIndDictColumn(prhs[9],nAtomsColumn,
                    nInitialIndexes);

			vIndDictRows = GetVIndDictRow(prhs[10],nInitialIndexes);

        } 

    // --->>

    /* Validate number of output (lhs) arguments*/    
    if(nlhs != 7)
    {
        mexErrMsgTxt("Wrong number of output arguments");
    }   
        
    /* Initialize Output variables */
    // <<--
        /* Initialize arrays for output data */
        
        /* Create mxArray for the approximation */
        double *mApprox = Set3DVar(plhs[0],nRows,nColumns,nColours);

        /* Create a dynamic array large enough to store the chosen column 
         * dictionary indexes - will not exceed nPixelsPlain
         */
        double *vIColumnAtomsBig = new double[nPixelsPlain];
        
        /* Create a dynamic array large enough to store the chosen row 
         * dictionary indexes
         */
        double *vIRowAtomsBig = new double[nPixelsPlain];        
        
        /* Create a dynamic array large enough to store the biorthogonal atoms */
        double *mBiorthBig = new double[nPixelsPlain*nPixelsPlain];
        
        /* Create a dynamic array large enough to store the chosen 
         * coefficients in the approximation
         */
		// Use nPixelsPlain as maxNAtoms may be too large 
        double *mCoeffBig = new double[nPixelsPlain*nColours];         
        
        /* Create a dynamic array large enough to store the orthogonal atoms */
        double *mOrthBig = new double[nPixelsPlain*nPixelsPlain];       
        
        /* Create a dynamic array large enough to store the chosen seperable atom 
         * indexes
         */
        double *vISepAtomsBig = new double[nPixelsPlain*nPixelsPlain];  
        
    // -->>
        
    /* # of Atoms chosen in target function */
    ulng nChosen = 0;       
              
    /* Invoke target function */
    OMPMlSep(mImage,mDictColumns,mDictRows,tolApprox,vWeights,maxNAtoms,
            vIndDictColumns,vIndDictRows,mApprox,vIColumnAtomsBig,
			vIRowAtomsBig,mBiorthBig,mCoeffBig,mOrthBig,vISepAtomsBig,
			nChosen,nInitialIndexes,nRows,nColumns,nColours,nAtomsColumn,
			nAtomsRow);    
         
    /* Initialize and populate remaining mxArrays now we have nChosen */
    // <<--
        double *vIColumnAtoms = Set2DVar(plhs[1],1,nChosen);
        
        /* Copy accros chosen indexes */
        CopyDbl(vIColumnAtomsBig,vIColumnAtoms,nChosen);  
        
        double *vIRowAtoms = Set2DVar(plhs[2],1,nChosen);
        
        /* Copy accros chosen indexes */
        CopyDbl(vIRowAtomsBig,vIRowAtoms,nChosen); 
        
        double *mBiorth = Set2DVar(plhs[3],nPixelsPlain,nChosen);
        
        /* Copy accros chosen biorthogonal atoms */
        CopyDbl(mBiorthBig,mBiorth,nChosen*nPixelsPlain); 
                                
        double *mCoeff = Set2DVar(plhs[4],nColours,nChosen);
        
        /* Copy accros chosen indexes */
        CopyDbl(mCoeffBig,mCoeff,nColours*nChosen);
        
        double *mOrth = Set2DVar(plhs[5],nPixelsPlain,nChosen);
        
        /* Copy accros chosen biorthogonal atoms */
        CopyDbl(mOrthBig,mOrth,nChosen*nPixelsPlain); 
        
        double *vISepAtoms = Set2DVar(plhs[6],1,nChosen);
        
        /* Copy accros chosen indexes */
        CopyDbl(vISepAtomsBig,vISepAtoms,nChosen);                  
    // -->>
    
    /* Free dynamicly assigned memory */
    // <<--
        delete [] vIRowAtomsBig;
        vIRowAtomsBig = 0;
        delete [] vIColumnAtomsBig;
        vIRowAtomsBig = 0;    
        delete [] mBiorthBig;
        mBiorthBig = 0;
        delete [] mCoeffBig;
        mCoeffBig = 0;
        delete [] mOrthBig;
        mOrthBig = 0;
        delete [] vISepAtomsBig;
        vISepAtomsBig = 0;
    // -->>        

}
